@@ -20,6 +20,7 @@ from filetype import guess_extension as filetype_guess_extension
 from .__version__ import __version__
 from .merge import merge_database
 from .tables import journals_table
+from .tables import list_columns
 from .tables import make_journals_table
 from .tables import make_settings_table
 from .tables import make_submissions_table
@@ -31,6 +32,7 @@ from .update import update_database
 
 Key = Union[str, int, float]
 Value = Union[str, int, float, None]
+Entry = Dict[str, Union[List[str], Value]]
 
 
 def guess_extension(file: bytes, default: str = "") -> str:
@@ -38,10 +40,10 @@ def guess_extension(file: bytes, default: str = "") -> str:
         return ""
     elif (file_type := filetype_guess_extension(file)) is None:
         return default
-    elif (file_ext := str(file_type)) in (exts := ("zip", "octet-stream")):
-        return default if default not in exts else file_ext
+    elif (ext := str(file_type)) in (exts := ("zip", "octet-stream")):
+        return default if default not in exts else ext
     else:
-        return file_ext
+        return ext
 
 
 def tiered_path(id_: Union[int, str], depth: int = 5, width: int = 2) -> str:
@@ -64,27 +66,28 @@ class FADatabaseTable:
         self.columns_info_: List[Tuple[str, str]] = []
         self.columns_: List[str] = []
         self.column_id_: str = ""
+        self.list_columns: List[str] = list_columns.get(table, [])
 
     def __len__(self) -> int:
         return self.database.connection.execute(f"SELECT COUNT(*) FROM {self.table}").fetchone()[0]
 
-    def __getitem__(self, key: Union[Key, Dict[str, Union[List[Value], Value]]]) -> Optional[Dict[str, Value]]:
+    def __getitem__(self, key: Union[Key, Entry]) -> Optional[Entry]:
         key = key if isinstance(key, dict) else {self.column_id: key}
-        entry: Optional[tuple] = self.select(key, self.columns).fetchone()
-        return {k.upper(): v for k, v in zip(self.columns, entry)} if entry is not None else None
+        return entry[0] if (entry := list(self.cursor_to_dict(self.select(key)))) else None
 
-    def __setitem__(self, key: Key, values: Dict[str, Value]):
-        values.update({self.column_id: key})
+    def __setitem__(self, key: Key, values: Entry):
+        values = {k.upper(): self.format_list(v) if isinstance(v, list) else v for k, v in values.items()}
+        values[self.column_id] = key
         self.insert(values)
 
     def __delitem__(self, key: Key):
         self.database.connection.execute(f"""DELETE FROM {self.table} WHERE {self.column_id} = ?""", (key,))
 
-    def __contains__(self, key: Union[Key, Dict[str, Value]]) -> bool:
+    def __contains__(self, key: Union[Key, Entry]) -> bool:
         return self[key] is not None
 
-    def __iter__(self) -> Generator[Dict[str, Value], None, None]:
-        return ({k.upper(): v for k, v in zip(self.columns_, entry)} for entry in self.select())
+    def __iter__(self) -> Generator[Entry, None, None]:
+        return self.cursor_to_dict(self.select())
 
     @property
     def columns_info(self) -> List[Tuple[str, str]]:
@@ -108,31 +111,45 @@ class FADatabaseTable:
         return self.column_id_
 
     def add_to_list(self, key: Key, values: Dict[str, List[Value]]) -> bool:
-        item: Optional[dict] = self[key]
-        if not item:
+        if not (values := {k.upper(): v for k, v in values.items() if v and k.upper() in self.list_columns}):
+            return False
+        elif not (item := self[key]):
             return False
         item = {k: item[k] for k in values.keys()}
-        item_new: Dict[str, str] = {k: ",".join(sorted(filter(bool, set(item[k].split(",") + v))))
-                                    for k, v in values.items()}
-        self.update(item_new, key) if item_new != item else None
+        item_new = {k: sorted(filter(bool, set(item[k] + v)), key=str.lower) for k, v in values.items()}
+        self.update({k: self.format_list(v) for k, v in item_new.items()}, key) if item_new != item else None
         return item_new != item
 
     def remove_from_list(self, key: Key, values: Dict[str, List[Value]]) -> bool:
-        item: Optional[dict] = self[key]
-        if not item:
+        if not (values := {k.upper(): v for k, v in values.items() if v and k.upper() in self.list_columns}):
+            return False
+        elif not (item := self[key]):
             return False
         item = {k: item[k] for k in values.keys()}
-        item_new: Dict[str, str] = {k: ",".join(sorted(filter(bool, set(item[k].split(",")) - set(v))))
-                                    for k, v in values.items()}
-        self.update(item_new, key) if item_new != item else None
+        item_new = {k: sorted(filter(bool, set(item[k]) - set(v)), key=str.lower) for k, v in values.items()}
+        self.update({k: self.format_list(v) for k, v in item_new.items()}, key) if item_new != item else None
         return item_new != item
 
     def reload(self):
         self.__init__(self.database, self.table)
 
-    def cursor_to_dict(self, cursor: Cursor, columns: List[str] = None) -> Generator[Dict[str, Value], None, None]:
-        columns = self.columns if columns is None else columns
-        return ({k.upper(): v for k, v in zip(columns, entry)} for entry in cursor)
+    def cursor_to_dict(self, cursor: Cursor, columns: List[str] = None) -> Generator[Entry, None, None]:
+        columns = list(map(str.upper, self.columns if columns is None else columns))
+        return ({k: self.unpack_list(v) if k in self.list_columns else v for k, v in zip(columns, entry)}
+                for entry in cursor)
+
+    @staticmethod
+    def format_list(obj: List[Value]) -> str:
+        return "".join(f"|{e}|" for e in sorted(set(map(str, obj)), key=str.lower))
+
+    @staticmethod
+    def unpack_list(obj: str) -> List[str]:
+        return [e for e in obj.split("|") if e]
+
+    def format_dict(self, obj: Entry) -> Dict[str, Value]:
+        obj = {k.upper().replace("_", ""): v for k, v in obj.items()}
+        obj = {k: self.format_list(v) if isinstance(v := obj.get(k, ""), list) else v for k in self.columns}
+        return obj
 
     def select(self, query: Dict[str, Union[List[Value], Value]] = None, columns: List[str] = None,
                query_and: bool = True, query_and_values: bool = False, like: bool = False,
@@ -194,7 +211,7 @@ class FADatabaseTable:
 
 class FADatabaseJournals(FADatabaseTable):
     def save_journal(self, journal: Dict[str, Union[int, str, list]]):
-        journal = {k.upper(): ",".join(map(str, v)) if isinstance(v, list) else v for k, v in journal.items()}
+        journal = self.format_dict(journal)
         self[journal["ID"]] = journal
 
     def add_mention(self, journal_id: int, user: str) -> bool:
@@ -223,29 +240,26 @@ class FADatabaseSettings(FADatabaseTable):
 
 
 class FADatabaseSubmissions(FADatabaseTable):
-    def save_submission_file(self, submission_id: int, file_ext: str, file: bytes) -> str:
+    def save_submission_file(self, submission_id: int, file: bytes, name: str, ext: str, guess_ext: bool = True) -> str:
         if not file:
             return ""
 
-        file_ext = guess_extension(file, file_ext)
-
+        ext = guess_extension(file, ext) if guess_ext else ext
         path = join(dirname(self.database.database_path), self.database.settings["FILESFOLDER"],
-                    tiered_path(submission_id), f"submission.{file_ext}")
-        makedirs(dirname(path), exist_ok=True)
-        with open(path, "wb") as f:
-            f.write(file)
+                    tiered_path(submission_id), name + f".{ext}" * bool(ext))
 
-        return file_ext
+        makedirs(dirname(path), exist_ok=True)
+        open(path, "wb").write(file)
+
+        return ext
 
     def save_submission(self, submission: Dict[str, Union[int, str, list]], file: Optional[bytes] = None):
-        submission = {k.upper(): ",".join(map(str, v)) if isinstance(v, list) else v for k, v in submission.items()}
-        submission = {k: submission.get(k, "") for k in {*submission.keys(), *self.columns}}
+        submission = self.format_dict(submission)
 
         submission["FILEEXT"] = name.split(".")[-1] if "." in (name := submission["FILELINK"].split("/")[-1]) else ""
+        submission["FILEEXT"] = self.save_submission_file(submission["ID"], file, "submission", submission["FILEEXT"])
         submission["FILESAVED"] = bool(file)
 
-        submission["FILEEXT"] = self.save_submission_file(submission["ID"], submission["FILEEXT"], file)
-
         self[submission["ID"]] = submission
 
     def set_folder(self, submission_id: int, folder: str) -> bool:
@@ -254,56 +268,45 @@ class FADatabaseSubmissions(FADatabaseTable):
         return folder_old != folder
 
     def add_favorite(self, submission_id: int, user: str) -> bool:
-        user = clean_username(user)
-        assert len(user) > 0, "User cannot be empty"
-        return self.add_to_list(submission_id, {"FAVORITE": [user]})
+        return self.add_to_list(submission_id, {"FAVORITE": [clean_username(user)]})
 
     def remove_favorite(self, submission_id: int, user: str) -> bool:
-        user = clean_username(user)
-        assert len(user) > 0, "User cannot be empty"
-        return self.remove_from_list(submission_id, {"FAVORITE": [user]})
+        return self.remove_from_list(submission_id, {"FAVORITE": [clean_username(user)]})
 
     def add_mention(self, submission_id: int, user: str) -> bool:
-        user = clean_username(user)
-        assert len(user) > 0, "User cannot be empty"
-        return self.add_to_list(submission_id, {"MENTIONS": [user]})
+        return self.add_to_list(submission_id, {"MENTIONS": [clean_username(user)]})
 
     def remove_mention(self, submission_id: int, user: str) -> bool:
-        user = clean_username(user)
-        assert len(user) > 0, "User cannot be empty"
-        return self.remove_from_list(submission_id, {"MENTIONS": [user]})
+        return self.remove_from_list(submission_id, {"MENTIONS": [clean_username(user)]})
 
 
 class FADatabaseUsers(FADatabaseTable):
     def new_user(self, user: str):
         user = clean_username(user)
         if user not in self:
-            self[user] = {f: "" for f in self.columns_}
+            self[user] = {f: "" for f in self.columns}
 
-    def enable_user(self, user: str):
-        user = clean_username(user)
-        if (user_entry := self[user]) is None:
+    def activate_user(self, user: str):
+        if (user_entry := self[(user := clean_username(user))]) is None:
             return
+        self.update({"FOLDERS": self.format_list([f.strip("!") for f in user_entry["FOLDERS"]])}, user)
 
-        self.update({"FOLDERS": ",".join([f.strip("!") for f in filter(bool, user_entry["FOLDERS"].split(","))])}, user)
-
-    def disable_user(self, user: str):
-        user = clean_username(user)
-        if (user_entry := self[user]) is None:
+    def deactivate_user(self, user: str):
+        if (user_entry := self[(user := clean_username(user))]) is None:
             return
-
-        self.update({"FOLDERS": ",".join([
-            f"!{f}" if f.lower() in ("gallery", "scraps", "favorites") else f
-            for f in filter(bool, user_entry["FOLDERS"].split(","))
-        ])}, user)
+        self.update({"FOLDERS": self.format_list([f"!{f.strip('!')}" for f in user_entry["FOLDERS"]])}, user)
 
     def add_user_folder(self, user: str, folder: str):
-        folder = folder.lower()
-        self.add_to_list(clean_username(user), {"FOLDERS": [folder]})
+        if not (user_entry := self[(user := clean_username(user))]):
+            return
+        elif (folder := folder.lower()) in user_entry["FOLDERS"]:
+            return
+        elif f"!{folder}" in user_entry["FOLDERS"]:
+            self.remove_user_folder(user, f"!{folder}")
+        self.add_to_list(user, {"FOLDERS": [folder]})
 
     def remove_user_folder(self, user: str, folder: str):
-        folder = folder.lower()
-        self.remove_from_list(clean_username(user), {"FOLDERS": [folder]})
+        self.remove_from_list(clean_username(user), {"FOLDERS": [folder.lower()]})
 
 
 class FADatabase:
