@@ -660,6 +660,63 @@ def make_database_4_7(db: Connection):
     db.commit()
 
 
+def make_database_4_8(db: Connection):
+    db.execute(
+        f"""CREATE TABLE IF NOT EXISTS USERS
+        (USERNAME TEXT UNIQUE NOT NULL CHECK (length(USERNAME) > 0),
+        FOLDERS TEXT NOT NULL,
+        PRIMARY KEY (USERNAME ASC));"""
+    )
+
+    db.execute(
+        """CREATE TABLE IF NOT EXISTS SUBMISSIONS
+        (ID INT UNIQUE NOT NULL CHECK (ID > 0),
+        AUTHOR TEXT NOT NULL CHECK (length(AUTHOR) > 0),
+        TITLE TEXT NOT NULL,
+        DATE DATE NOT NULL CHECK (DATE==strftime('%Y-%m-%d',DATE)),
+        DESCRIPTION TEXT NOT NULL,
+        TAGS TEXT NOT NULL,
+        CATEGORY TEXT NOT NULL,
+        SPECIES TEXT NOT NULL,
+        GENDER TEXT NOT NULL,
+        RATING TEXT NOT NULL,
+        FILEURL TEXT NOT NULL,
+        FILEEXT TEXT NOT NULL,
+        FILESAVED INT NOT NULL CHECK (FILESAVED in (0, 1)),
+        FAVORITE TEXT NOT NULL,
+        MENTIONS TEXT NOT NULL,
+        FOLDER TEXT NOT NULL CHECK (FOLDER = 'gallery' OR FOLDER = 'scraps'),
+        USERUPDATE INT NOT NULL CHECK (USERUPDATE in (0, 1)),
+        PRIMARY KEY (ID ASC));"""
+    )
+
+    db.execute(
+        """CREATE TABLE IF NOT EXISTS JOURNALS
+        (ID INT UNIQUE NOT NULL CHECK (ID > 0),
+        AUTHOR TEXT NOT NULL CHECK (length(AUTHOR) > 0),
+        TITLE TEXT NOT NULL,
+        DATE DATE NOT NULL CHECK (DATE==strftime('%Y-%m-%d',DATE)),
+        CONTENT TEXT NOT NULL,
+        MENTIONS TEXT NOT NULL,
+        USERUPDATE INT NOT NULL CHECK (USERUPDATE in (0, 1)),
+        PRIMARY KEY (ID ASC));""",
+    )
+
+    db.execute(
+        """CREATE TABLE IF NOT EXISTS SETTINGS
+        (SETTING TEXT UNIQUE NOT NULL CHECK (length(SETTING) > 0),
+        SVALUE TEXT NOT NULL CHECK (length(SVALUE) > 0),
+        PRIMARY KEY (SETTING ASC));"""
+    )
+
+    db.execute(f"INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES (?, ?)", ["HISTORY", "[]"])
+    db.execute(f"INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES (?, ?)", ["COOKIES", "{}"])
+    db.execute(f"INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES (?, ?)", ["FILESFOLDER", "FA.files"])
+    db.execute(f"INSERT OR IGNORE INTO SETTINGS (SETTING, SVALUE) VALUES (?, ?)", ["VERSION", "4.8.0"])
+
+    db.commit()
+
+
 def update_2_7_to_3(db: Connection) -> Connection:
     from .database import tiered_path
 
@@ -1392,6 +1449,77 @@ def update_4_6_to_4_7(db: Connection) -> Connection:
     return connect_database(db_path)
 
 
+def update_4_7_to_4_8(db: Connection) -> Connection:
+    print("Updating 4.7.0 to 4.8.0")
+    db_new: Optional[Connection] = None
+
+    db_path: str = dp if (dp := database_path(db)) else "FA.db"
+    db_new_path: str = path_join(dirname(db_path), "new_" + basename(db_path))
+
+    try:
+        db_new = connect_database(db_new_path)
+        make_database_4_8(db_new)
+
+        # Transfer common submissions and users data
+        print("Transfer common submissions and users data")
+        db.execute(f"ATTACH DATABASE '{db_new_path}' AS db_new")
+        db.execute(
+            """INSERT OR IGNORE INTO db_new.USERS
+            SELECT * FROM USERS"""
+        )
+        db.execute(
+            """INSERT OR IGNORE INTO db_new.SUBMISSIONS
+            SELECT * FROM SUBMISSIONS"""
+        )
+        db.execute(
+            """INSERT OR IGNORE INTO db_new.JOURNALS
+            SELECT * FROM JOURNALS"""
+        )
+        db.execute(
+            """INSERT OR IGNORE INTO db_new.SETTINGS
+            SELECT * FROM SETTINGS WHERE SETTING NOT IN ('VERSION')"""
+        )
+
+        db.commit()
+        db.close()
+        db = None
+
+        for u, fs in db_new.execute("select USERNAME, FOLDERS from USERS"):
+            fs = "".join(sorted(f"|{f}|" for f in fs.split(",") if f))
+            db_new.execute("update USERS set FOLDERS = ? where USERNAME = ?", (fs, u))
+        db_new.commit()
+        for i, ts, fs, ms in db_new.execute("select ID, TAGS, FAVORITE, MENTIONS from SUBMISSIONS"):
+            if not ts and not fs and not ms:
+                continue
+            ts = "".join(sorted(f"|{t}|" for t in ts.split(",") if t))
+            fs = "".join(sorted(f"|{f}|" for f in fs.split(",") if f))
+            ms = "".join(sorted(f"|{m}|" for m in ms.split(",") if m))
+            db_new.execute("update SUBMISSIONS set TAGS = ?, FAVORITE = ?, MENTIONS = ? where ID = ?", (ts, fs, ms, i))
+        db_new.commit()
+        for i, ms in db_new.execute("select ID, MENTIONS from JOURNALS where MENTIONS != ''"):
+            if not ms:
+                continue
+            ms = "".join(sorted(f"|{m}|" for m in ms.split(",") if m))
+            db_new.execute("update JOURNALS set MENTIONS = ? where ID = ?", (ms, i))
+        db_new.commit()
+
+        db_new.commit()
+        db_new.close()
+        move(db_path, path_join(dirname(db_path), "v4_7" + basename(db_path)))
+        move(db_new_path, db_path)
+    except (BaseException, Exception) as err:
+        print("Database update interrupted!")
+        if db is not None:
+            db.commit()
+            db.close()
+        if db_new is not None:
+            db_new.commit()
+            db_new.close()
+        raise err
+
+    return connect_database(db_path)
+
+
 def update_version(db: Connection, version: str, target_version: str) -> Connection:
     print(f"Updating {version} to {target_version}")
     db.execute("UPDATE SETTINGS SET SVALUE = ? WHERE SETTING = 'VERSION'", [target_version])
@@ -1406,35 +1534,37 @@ def update_database(db: Connection, version: str) -> Connection:
         return db
     elif v > 0:
         raise DatabaseError("Database version is newer than program.")
-    elif (v := compare_versions(db_version, "2.7.0")) < 0:
+    elif compare_versions(db_version, "2.7.0") < 0:
         raise DatabaseError("Update does not support versions lower than 2.11.2")
-    elif v >= 0 > (v := compare_versions(db_version, "3.0.0")):
+    elif compare_versions(db_version, "3.0.0") < 0:
         return update_database(update_2_7_to_3(db), version)  # 2.7.x to 3.0.0
-    elif v >= 0 > (v := compare_versions(db_version, "3.1.0")):
+    elif compare_versions(db_version, "3.1.0") < 0:
         return update_database(update_3_to_3_1(db), version)  # 3.0.x to 3.1.0
-    elif v >= 0 > (v := compare_versions(db_version, "3.2.0")) < 0:
+    elif compare_versions(db_version, "3.2.0") < 0:
         return update_database(update_3_1_to_3_2(db), version)  # 3.1.x to 3.2.0
-    elif v >= 0 > (v := compare_versions(db_version, "3.3.0")) < 0:
+    elif compare_versions(db_version, "3.3.0") < 0:
         return update_database(update_3_2_to_3_3(db), version)  # 3.2.x to 3.3.0
-    elif v >= 0 > (v := compare_versions(db_version, "3.4.0")) < 0:
+    elif compare_versions(db_version, "3.4.0") < 0:
         return update_database(update_version(db, db_version, "3.4.0"), version)  # 3.3.x to 3.4.0
-    elif v >= 0 > (v := compare_versions(db_version, "3.5.0")) < 0:
+    elif compare_versions(db_version, "3.5.0") < 0:
         return update_database(update_3_4_to_3_5(db), version)  # 3.4.x to 3.5.0
-    elif v >= 0 > (v := compare_versions(db_version, "3.8.0")) < 0:
+    elif compare_versions(db_version, "3.8.0") < 0:
         return update_database(update_version(db, db_version, "3.8.0"), version)  # 3.5.0-3.7.x to 3.8.0
-    elif v >= 0 > (v := compare_versions(db_version, "4.0.0")) < 0:
+    elif compare_versions(db_version, "4.0.0") < 0:
         return update_database(update_3_8_to_4(db), version)  # 3.8.x to 4.0.0
-    elif v >= 0 > (v := compare_versions(db_version, "4.3.0")) < 0:
+    elif compare_versions(db_version, "4.3.0") < 0:
         return update_database(update_4_2_to_4_3(db), version)  # 4.0.0-4.2.x to 4.3.0
-    elif v >= 0 > (v := compare_versions(db_version, "4.4.0")) < 0:
+    elif compare_versions(db_version, "4.4.0") < 0:
         return update_database(update_4_3_to_4_4(db), version)  # 4.3.x to 4.4.0
-    elif v >= 0 > (v := compare_versions(db_version, "4.5.0")) < 0:
+    elif compare_versions(db_version, "4.5.0") < 0:
         return update_database(update_4_4_to_4_5(db), version)  # 4.4.x to 4.5.0
-    elif v >= 0 > (v := compare_versions(db_version, "4.6.0")) < 0:
+    elif compare_versions(db_version, "4.6.0") < 0:
         return update_database(update_4_5_to_4_6(db), version)  # 4.5.x to 4.6.0
-    elif v >= 0 > (v := compare_versions(db_version, "4.7.0")) < 0:
+    elif compare_versions(db_version, "4.7.0") < 0:
         return update_database(update_4_6_to_4_7(db), version)  # 4.6.x to 4.7.0
-    elif v >= 0 > compare_versions(db_version, version) < 0:
+    elif compare_versions(db_version, "4.8.0") < 0:
+        return update_database(update_4_7_to_4_8(db), version)  # 4.7.x to 4.8.0
+    elif compare_versions(db_version, version) < 0:
         return update_version(db, db_version, version)  # Update to latest patch
 
     return db
